-- TODO: Handle colors
local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0")

local Boss = RW.bossPrototype
local timerProt = {}
local mt = {__index = timerProt}

local Party = RW:GetPlugin("Raid")
local BITS = RW.BITS

Boss.BITS = Boss.BITS or {}
Boss.BITS.BAR = {
	CD = 0x00000001,
	DUR = 0x00000002,
	CAST = 0x00000004,
	NORMAL = 0x00000008,
	DUR_TARGET = 0x00000010,
	DUR_TARGET_STACK = 0x00000020,
	DUR_STACK = 0x00000040,
	BERSERK = 0x00000080,
}

local custText = true
function Boss:_Timer(tType, time, text, icon, optionName, default, color, endMessage, customText)
	if type(text) == "number" then 
		custText = customText or false
		return self:_Timer(tType, time, GetSpellInfo(text), icon or text, optionName or text, default, color, endMessage, customText) 	
	end
	if type(icon) == "number" then return self:_Timer(tType, time, text, select(3, GetSpellInfo(icon)), optionName, default, color, endMessage, customText) end
	
	if not optionName then optionName = text end
	if type(optionName) == "number" then
		self:AddSpellOption("General", optionName)
		optionName = RW.SpellName[optionName]
	else
		self:AddGeneralOption(optionName)
	end
	
	local defaultOption = self.defaultOptions[optionName]
	local option = self.options[optionName]
	
	-- Add the default state
	if ((default == nil) or default) then
		defaultOption.uses = RW:AddBit(defaultOption.uses, BITS.BAR)
		defaultOption.BAR = RW:AddBit(defaultOption.BAR, self.BITS.BAR[tType])
	else
		defaultOption.BAR = RW:AddBit(defaultOption.BAR, 0)
	end
	defaultOption.endMessage = endMessage or false
	-- Add settings for so that UI can draw them no matter if they are default on or off
	option.BAR = RW:AddBit(option.BAR, self.BITS.BAR[tType])
	option.uses = RW:AddBit(option.uses, BITS.BAR)
	
	local obj =  setmetatable({
		type = tType,
		time = time,
		text = text,
		customText = custText,
		icon = icon,
		mod = self,
		id = optionName,
		running = {},
		scheduled = {},
		default = default
	}, mt)
	
	self.timers = self.timers or {}
	tinsert(self.timers, obj)
	
	custText = true
	
	return obj
end

function Boss:Timer(...)
	return self:_Timer("NORMAL", ...)
end

function Boss:CDTimer(...)
	return self:_Timer("CD", ...)
end

function Boss:DurTimer(...)
	return self:_Timer("DUR", ...)
end

function Boss:DurTargetTimer(...)
	return self:_Timer("DUR_TARGET", ...)
end

function Boss:DurTargetStackTimer(...)
	return self:_Timer("DUR_TARGET_STACK", ...)
end

function Boss:DurStackTimer(...)
	return self:_Timer("DUR_STACK", ...)
end

function Boss:BerserkTimer(time, text, ...)
	return self:_Timer("BERSERK", time, text or "Berserk", ...)
end

function Boss:CastTimer(time, ...)
	if time > 1000 then
		local castTime = select(7, GetSpellInfo(time))
		local spellHaste = select(7, GetSpellInfo(53142)) / 10000
		local spell = time
		time = castTime/spellHaste/1000
		return self:CastTimer(time, spell, ...)
	end
	return self:_Timer("CAST", time, ...)
end

function Boss:StopAllTimers()
	if not self.timers then return end
	for i, timer in pairs(self.timers) do
		for j, runningID in pairs(timer.running) do
			RW:SendMessage("RW_CancelBar", runningID)
			timer.running[j] = nil
		end
	end
end

function Boss:RW_BarFinished(message, id, text, icon)
	if not self.timers then return end
	for i, timer in pairs(self.timers) do
		for j, runningID in pairs(timer.running) do
			if runningID == id then
				if self.db.profile[timer.id].endMessage then
					local c = timer.color or {r=1, g=1, b=1}
					self:SendMessage("RW_Message", text, c.r, c.g, c.b, icon)
				end
				timer.running[j] = nil
			end
		end
	end
end

function timerProt:Start(time, info, ...)
	if bit.band(self.mod.db.profile[self.id].uses, RW.BITS.BAR) ~= RW.BITS.BAR then return end
	if bit.band(self.mod.db.profile[self.id].BAR, self.mod.BITS.BAR[self.type]) ~= self.mod.BITS.BAR[self.type] then return end
	if time and type(time) ~= "number" then return self:Start(nil, time, info, ...) end
	
	local text
	if self.customText then		
		text = RW:Format(self.text, info, ...)
	else
		text = RW:Format(L["FORMAT_BAR_"..self.type], self.text, info, ...)	
	end
	text = text:gsub(">([%S\s]+)<", function(name)
		if Party.group[name] then
			local color = RAID_CLASS_COLORS[Party.group[name].class] or self.color
			name = ("|cff%.2x%.2x%.2x%s|r"):format(color.r * 255, color.g * 255, color.b * 255, name)
		end
		return name
	end)
	
	RW:SendMessage("RW_StartBar", self.id..(info or ""), time or self.time, text, self.icon, self.mod.db.profile[self.id].Color)
	tinsert(self.running, self.id..(info or ""))
end

function timerProt:Stop(extraID)
	if extraID then
		RW:SendMessage("RW_CancelBar", self.id..extraID)
	else
		for i, id in pairs(self.running) do
			RW:SendMessage("RW_CancelBar", id)
		end
	end
end
timerProt.Cancel = timerProt.Stop

local function startTimer(timer, ...)
	timer:Start(...)
end

function timerProt:Schedule(stime, time, info, ...)
	if time and type(time) ~= "number" then return self:Schedule(stime, nil, time, info, ...) end
	self.mod:Schedule(self.id..(info or ""), stime, startTimer, self, time, info, ...)
	tinsert(self.scheduled, self.id..(info or ""))
end

function timerProt:ScheduleRepeating(stime, now, time, info, ...)
	if time and type(time) ~= "number" then return self:ScheduleRepeating(stime, now, nil, time, info, ...) end
	self.mod:ScheduleRepeating(self.id..(info or ""), stime, startTimer, now, self, time, info, ...)
	tinsert(self.scheduled, self.id..(info or ""))
end

function timerProt:Unschedule(extraID)
	if not extraID then
		for i, scheduleID in pairs(self.scheduled) do
			self.mod:Unschedule(scheduleID)
			self.scheduled[i] = nil
		end
	else
		self.mod:Unschedule(self.id..(extraID or ""))
		RW:TRemove(self.scheduled, self.id..(extraID or ""))
	end
end

-- Options
Boss.optionGets = Boss.optionGets or {}
function Boss.optionGets.BAR(self, option)
	local AceGUI = LibStub("AceGUI-3.0")
	local group = AceGUI:Create("InlineGroup")
	group:SetFullWidth(true)
	group:SetTitle("Options")
	
	for bType, value in pairs(self.BITS.BAR) do
		if bit.band(self.options[option.text].BAR, value) == value then
			local check = AceGUI:Create("CheckBox")
			check:SetLabel(L["BAR_"..bType]) -- TODO: Set more info in the text, (the actual time for the bar)
			check:SetValue(bit.band(self.db.profile[option.text].BAR, value) == value)
			check:SetUserData("option", option)
			check:SetUserData("type", "BAR")
			check:SetUserData("subType", bType)
			check:SetUserData("boss", self)
			check:SetCallback("OnValueChanged", self.optionHelpers.onValueChanged)
			check:SetCallback("OnEnter", self.optionHelpers.onOptionEnter)
			check:SetCallback("OnLeave", self.optionHelpers.onOptionLeave)
			
			group:AddChild(check)
		end
	end
	return group
end 