local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0")

local LSM = LibStub("LibSharedMedia-3.0")
LSM:Register("sound", "RunAwayLittleGirl", [[Sound\Creature\HoodWolf\HoodWolfTransformPlayer01.wav]])
LSM:Register("sound", "Tata", [[Sound\Spells\PVPFlagTaken.wav]])

local Boss = RW.bossPrototype
local warningProt = {}
local mt = {__index = warningProt}

local Party = RW:GetPlugin("Raid")
local BITS = RW.BITS

Boss.BITS = Boss.BITS or {}
Boss.BITS.WARNING = {
	NORMAL	= 0x00000001,
	CAST 	= 0x00000002,
	RUNAWAY = 0x00000004,
}

local custText = true
function Boss:_Warning(wType, text, icon, optionName, default, color, sound)
	if type(text) == "number" then 
		custText = customText or false
		return self:_Warning(wType, GetSpellInfo(text), text, text, icon, optionName, color) end
	if type(icon) == "number" then return self:_Warning(wType, text, select(3, GetSpellInfo(icon)), optionName, default, color, sound) end

	if not optionName then optionName = text end
	if type(optionName) == "number" then
		self:AddSpellOption("General", optionName)
		optionName = RW.SpellName[optionName]
	else
		self:AddGeneralOption(optionName)
	end
	
	local defaultOption = self.defaultOptions[optionName]
	local option = self.options[optionName]
	
	-- Add the default state
	if ((default == nil) or default) then
		defaultOption.uses = RW:AddBit(defaultOption.uses, BITS.WARNING)
		defaultOption.WARNING = RW:AddBit(defaultOption.WARNING, self.BITS.WARNING[wType])
	else
		defaultOption.WARNING = RW:AddBit(defaultOption.WARNING, 0)
	end
	defaultOption.endMessage = endMessage or false
	-- Add settings for so that UI can draw them no matter if they are default on or off
	option.WARNING = RW:AddBit(option.WARNING, self.BITS.WARNING[wType])
	option.uses = RW:AddBit(option.uses, BITS.WARNING)
	
	if wType == "RUNAWAY" and not sound 
		then defaultOption.Sound = "RunAwayLittleGirl"
	else
		defaultOption.Sound = "Tata"
	end
	
	local obj = setmetatable({
		type = wType,
		text = text,
		customText = custText,
		icon = icon,
		id = optionName,
		color = color,
		mod = self,
	}, mt)
	
	self.warnings = self.warnings or {}
	tinsert(self.warnings, obj)
	return obj
end

function Boss:Warning(...)
	return self:_Warning("NORMAL", ...)
end

function Boss:RunawayWarning(...)
	return self:_Warning("RUNAWAY", ...)
end

function warningProt:Show(...)
	if bit.band(self.mod.db.profile[self.id].uses, RW.BITS.WARNING) ~= RW.BITS.WARNING then return end
	if bit.band(self.mod.db.profile[self.id].WARNING, self.mod.BITS.WARNING[self.type]) ~= self.mod.BITS.WARNING[self.type] then return end
	self.mod:SendMessage("RW_Warning", self.text, self.icon, self.color)
	if self.mod.db.profile[self.id].Sound then PlaySoundFile(LSM:Fetch("sound", self.mod.db.profile[self.id].Sound)) end
end

-- Options
Boss.optionGets = Boss.optionGets or {}
function Boss.optionGets.WARNING(self, option)
	local AceGUI = LibStub("AceGUI-3.0")
	local group = AceGUI:Create("InlineGroup")
	group:SetFullWidth(true)
	group:SetTitle("Options")
	
	local drop = AceGUI:Create("LSM30_Sound")
	drop:SetList(LSM:HashTable("sound"))
	drop:SetLabel("Sound")
	drop:SetUserData("option", option)
	drop:SetUserData("boss", self)
	drop:SetUserData("key", "Sound")
	drop:SetCallback("OnValueChanged", self.optionHelpers.onValueChanged2)
	drop:SetValue(self.db.profile[option.text].Sound)
	group:AddChild(drop)
	
	for bType, value in pairs(self.BITS.WARNING) do
		if bit.band(self.options[option.text].WARNING, value) == value then
			local check = AceGUI:Create("CheckBox")
			check:SetLabel(L["WARNING_"..bType]) -- TODO: Set more info in the text, (the actual time for the WARNING)
			check:SetValue(bit.band(self.db.profile[option.text].WARNING, value) == value)
			check:SetUserData("option", option)
			check:SetUserData("type", "WARNING")
			check:SetUserData("subType", bType)
			check:SetUserData("boss", self)
			check:SetCallback("OnValueChanged", self.optionHelpers.onValueChanged)
			check:SetCallback("OnEnter", self.optionHelpers.onOptionEnter)
			check:SetCallback("OnLeave", self.optionHelpers.onOptionLeave)
			group:AddChild(check)
		end
	end
	return group
end 